from collections import defaultdict
from dataclasses import dataclass
from enum import Enum
from random import shuffle, randrange
from typing import List, Dict

from model.animals import random_animal_name


class Value(Enum):
    Ace = 1
    Two = 2
    Three = 3
    Four = 4
    Five = 5
    Six = 6
    Seven = 7
    Eight = 8
    Nine = 9
    Ten = 10
    Jack = 11
    Queen = 12
    King = 13


class Color(Enum):
    Hearts = "♥"
    Spades = "♠"
    Clubs = "♣"
    Diamonds = "♦"


@dataclass(frozen=True)
class Card:
    value: Value
    color: Color

    def __str__(self) -> str:
        return f"{self.color.value} {self.value.name} {self.color.value}"


class Deck:
    def __init__(self):
        self.cards = [Card(v, c) for v in Value for c in Color]
        self.defausse = []

    def __len__(self):
        return len(self.cards)

    def random_card(self) -> Card:
        if not self.cards:
            self.reset()

        card = self.cards.pop(randrange(len(self.cards)))
        self.defausse.append(card)
        return card

    def reset(self):
        self.cards.extend(self.defausse)
        self.defausse.clear()
        shuffle(self.cards)


class Player:
    name: str = random_animal_name()
    hand: List[Card] = []

    def give(self, card: Card):
        self.hand.append(card)


class Game:
    def __init__(self, deck: Deck = Deck(), players=None):
        if players is None:
            players = []

        self.deck: Deck = deck
        self.players: List[Player] = players
        self.defeats: Dict[Player, int] = defaultdict(lambda: 0)

    def new_game(self):
        self.deck.reset()
        while len(self.players) > 1:
            self.new_turn()
        winner = self.players[0]
        print(f"Game over - Player {winner.name} wins with {len(winner.hand)} cards!")
        pass

    def new_turn(self):
        # Distribution
        shuffle(self.deck)
        for p in self.players:
            p.give(self.deck.random_card())

        # Tour
        for p in self.players:
            announce = p.announce(current_bet)




        # TODO: Put next first player first of list