from unittest import TestCase

from model.data import Deck, Player, Card, Value, Color


class TestDeck(TestCase):

    def setUp(self) -> None:
        super().setUp()
        self.deck = Deck()

    def testInitDeck(self):
        self.assertEqual(52, len(self.deck), "52 cards")
        self.assertEqual(52, len(set(self.deck.cards)), "unique cards")

    def testRandomCard(self):
        cards = []
        for i in range(len(self.deck)):
            cards.append(self.deck.random_card())

        self.assertEqual(52, len(set(cards)), "draw each card once")


class TestPlayer(TestCase):

    def setUp(self) -> None:
        super().setUp()
        self.player = Player()

    def testHand(self):
        self.assertEqual(0, len(self.player.hand), "Begin no cards")
        self.assertTrue(len(self.player.name), "Has a name")

        self.player.give(Card(Value.Ace, Color.Hearts))

        self.assertEqual(1, len(self.player.hand), "Gave one card")
        self.assertEqual(Value.Ace, self.player.hand[0].value, "Is Ace")
        self.assertEqual(Color.Hearts, self.player.hand[0].color, "of Hearts")

    def testDefeats(self):
        pass
